﻿/*
  In App.xaml:
  <Application.Resources>
      <vm:ViewModelLocatorTemplate xmlns:vm="clr-namespace:httpExplorer.ViewModel"
                                   x:Key="Locator" />
  </Application.Resources>
  
  In the View:
  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
  
  OR (WPF only):
  
  xmlns:vm="clr-namespace:httpExplorer.ViewModel"
  DataContext="{Binding Source={x:Static vm:ViewModelLocatorTemplate.ViewModelNameStatic}}"
*/

using GalaSoft.MvvmLight;
using httpExplorer.Model;

namespace httpExplorer.ViewModel
{
    /// <summary>
    /// This class contains static references to all the view models in the
    /// application and provides an entry point for the bindings.
    /// <para>
    /// Use the <strong>mvvmlocatorproperty</strong> snippet to add ViewModels
    /// to this locator.
    /// </para>
    /// <para>
    /// In Silverlight and WPF, place the ViewModelLocatorTemplate in the App.xaml resources:
    /// </para>
    /// <code>
    /// &lt;Application.Resources&gt;
    ///     &lt;vm:ViewModelLocatorTemplate xmlns:vm="clr-namespace:httpExplorer.ViewModel"
    ///                                  x:Key="Locator" /&gt;
    /// &lt;/Application.Resources&gt;
    /// </code>
    /// <para>
    /// Then use:
    /// </para>
    /// <code>
    /// DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"
    /// </code>
    /// <para>
    /// You can also use Blend to do all this with the tool's support.
    /// </para>
    /// <para>
    /// See http://www.galasoft.ch/mvvm/getstarted
    /// </para>
    /// <para>
    /// In <strong>*WPF only*</strong> (and if databinding in Blend is not relevant), you can delete
    /// the Main property and bind to the ViewModelNameStatic property instead:
    /// </para>
    /// <code>
    /// xmlns:vm="clr-namespace:httpExplorer.ViewModel"
    /// DataContext="{Binding Source={x:Static vm:ViewModelLocatorTemplate.ViewModelNameStatic}}"
    /// </code>
    /// </summary>
    public class ViewModelLocator
    {
        private static ModelLocator _modelLocator = new ModelLocator();
        public static ModelLocator ModelLocator
        {
            get { return _modelLocator; }
            set { _modelLocator = value; }
        }

        /// <summary>
        /// Initializes a new instance of the ViewModelLocator class.
        /// </summary>
        public ViewModelLocator()
        {
            if (ViewModelBase.IsInDesignModeStatic)
            {
                // Create design time view models
            }
            else
            {
                // Create run time view models
                //_model = new ModelLocator();

            }

            CreateMain();
        }

        private static InputViewModel _inputViewModel;
        /// <summary>
        /// Gets the Main property.
        /// </summary>
        public static InputViewModel InputViewModelStatic
        {
            get
            {
                if (_inputViewModel == null)
                {
                    CreateMain();
                }

                return _inputViewModel;
            }
        }



        private static OutputViewModel _outputViewModel;


        /// <summary>
        /// Gets the Main property.
        /// </summary>
        public static OutputViewModel OutputViewModelStatic
        {
            get
            {
                if (_outputViewModel == null)
                {
                    CreateMain();
                }

                return _outputViewModel;
            }
        }

        /// <summary>
        /// Gets the OutputViewModel property.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance",
            "CA1822:MarkMembersAsStatic",
            Justification = "This non-static member is needed for data binding purposes.")]
        public OutputViewModel OutputViewModel
        {
            get
            {
                return OutputViewModelStatic;
            }
        }



        /// <summary>
        /// Gets the InputViewModel property.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance",
            "CA1822:MarkMembersAsStatic",
            Justification = "This non-static member is needed for data binding purposes.")]
        public InputViewModel InputViewModel
        {
            get
            {
                return InputViewModelStatic;
            }
        }



        private static NetworkSettingsViewModel _networkSettingsViewModel;


        /// <summary>
        /// Gets the Main property.
        /// </summary>
        public static NetworkSettingsViewModel NetworkSettingsViewModelStatic
        {
            get
            {
                if (_networkSettingsViewModel == null)
                {
                    CreateMain();
                }

                return _networkSettingsViewModel;
            }
        }

        /// <summary>
        /// Gets the NetworkSettingsViewModel property.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance",
            "CA1822:MarkMembersAsStatic",
            Justification = "This non-static member is needed for data binding purposes.")]
        public NetworkSettingsViewModel NetworkSettingsViewModel
        {
            get
            {
                return NetworkSettingsViewModelStatic;
            }
        }


        /// <summary>
        /// Provides a deterministic way to delete the Main property.
        /// </summary>
        public static void ClearMain()
        {
            _inputViewModel.Cleanup();
            _inputViewModel = null;
            _outputViewModel.Cleanup();
            _outputViewModel = null;
            _networkSettingsViewModel.Cleanup();
            _networkSettingsViewModel = null;
        }

        /// <summary>
        /// Provides a deterministic way to create the Main property.
        /// </summary>
        public static void CreateMain()
        {
            if (_inputViewModel == null)
            {
                _inputViewModel = new InputViewModel();
            }
            if (_outputViewModel == null)
            {
                _outputViewModel = new OutputViewModel();
            }
            if (_networkSettingsViewModel == null)
            {
                _networkSettingsViewModel = new NetworkSettingsViewModel();
            }
        }

        /// <summary>
        /// Cleans up all the resources.
        /// </summary>
        public static void Cleanup()
        {
            ClearMain();
        }
    }
}